#!/bin/env perl

use strict;
use File::Basename qw ( dirname );
use feature qw(say);

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Config qw( StoreType BusinessUnits StoreTypesWithCartridge );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportAttributes );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

sub Main {
    local $| = 1;

    chdir dirname $0;

    my $SourceDir = '../FileChanges/srv/epages/eproot/Cartridges/DE_EPAGES';
    return unless -d $SourceDir;
    chdir $SourceDir;

    my ($Stores, $AttributeFiles, $ObjectFiles) = _GetImports();

    foreach my $Store (@$Stores) {
        RunOnStore(
            Store => $Store,
            Sub => sub  {
                foreach (@{$AttributeFiles->{$Store}}) {
                    say qq(ImportAttributes($_, '/', $Store));
                    ImportAttributes($_, '/');
                }
                foreach (@{$ObjectFiles->{$Store}}) {
                    say qq(ImportObjects($_, '/', $Store));
                    ImportObjects($_, '/');
                }
            },
        );
    }

}

sub _GetImports {
    my $BU;
    push @{$BU->{StoreType($_)}}, $_ foreach @{_GetBusinessUnits()};

    my ($UsedLanguages, $AttributeFiles, $ObjectFiles) = ({}, {}, {});
    foreach my $Dir (sort glob '*/Database/XML') {
        my $Cartridge = $Dir;
        $Cartridge =~ s,/.*,,;

        foreach my $Store (@{_StoresWithCartridge($Cartridge, $BU)}) {
            $UsedLanguages->{$Store} = _GetUsedLanguages($Store)
                unless defined $UsedLanguages->{$Store};
            foreach my $XMLFile (sort glob "$Dir/*.xml") {
                if( $XMLFile =~ /\bTranslation\.(\w+\.)+(\w\w|\w\w-\w\w)\.xml$/ ) {
                    next unless $2 ~~ @{$UsedLanguages->{$Store}};
                    push @{$AttributeFiles->{$Store}}, "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/$XMLFile";
                } elsif( $XMLFile !~ /\bHooks[^\/]*\.xml$/ ) {
                    push @{$ObjectFiles->{$Store}}, "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/$XMLFile";
                }
            }
        }
    }
    return ([sort keys %$UsedLanguages], $AttributeFiles, $ObjectFiles);
}

sub _GetBusinessUnits {
    return [ sort split /\s+/, $ENV{EPAGES_BUSINESS_UNITS} ]
        if $ENV{EPAGES_BUSINESS_UNITS} =~ /\S/;
    return [ sort keys %{BusinessUnits()} ];
}

sub _GetUsedLanguages ($) {
    my ($Store) = @_;
    my (@Languages);

    my @SelectUsedLanguages = ('SELECT code2 FROM language WHERE isused = 1',
                               'SELECT code2,dialect FROM language WHERE isused = 1');

    RunOnStore( 'Store'=> $Store, 'Sub' => sub {
        my $dbi = GetCurrentDBHandle();
        my $select = $SelectUsedLanguages[$dbi->existsTableColumn('language', 'dialect')];
        foreach my $Row (@{$dbi->execute($select)}) {
            my $Lang = $Row->[0];
            $Lang .= "-$Row->[1]" if defined $Row->[1];
            push @Languages, $Lang;
        }
    });
    return \@Languages;
}

sub _StoresWithCartridge ($$) {
    my ($Cartridge, $BU) = @_;

    my @Stores;
    foreach my $StoreType (@{StoreTypesWithCartridge("DE_EPAGES::$Cartridge")}) {
        push @Stores, $_ foreach @{$BU->{$StoreType}};
    }
    return \@Stores;
}

RunScript(
    'Sub' => \&Main
);
